@@ -159,6 +159,10 @@ extern "C" {
 #  define MRN_HAVE_HA_EXTRA_DISABLE_UNIQUE_RECORD_FILTER
 #endif
 
+#if MYSQL_VERSION_ID >= 80017 && !defined(MRN_MARIADB_P)
+#  define MRN_HAVE_MYSQL_TYPE_TYPED_ARRAY
+#endif
+
 #if MYSQL_VERSION_ID >= 50604 && !defined(MRN_MARIADB_P)
 #  define MRN_TIMESTAMP_USE_TIMEVAL
 #elif defined(MRN_MARIADB_P)
@@ -455,6 +459,23 @@ typedef uint mrn_alter_table_flags;
 #  define MRN_HANDLER_HAVE_HA_EXTRA
 #endif
 
+#if defined(MRN_MARIADB_P) && \
+   ((MYSQL_VERSION_ID >= 100320 && MYSQL_VERSION_ID < 100400) || \
+    (MYSQL_VERSION_ID >= 100410))
+#  define MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE mrn_override
+#  define MRN_HANDLER_HAVE_RESTORE_AUTO_INCREMENT_NO_ARGUMENT
+#  define MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE mrn_override
+#else
+#  define MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE
+#  define MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE
+#endif
+
+#if MYSQL_VERSION_ID >= 50700 && !defined(MRN_MARIADB_P)
+#  define MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST const
+#else
+#  define MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST
+#endif
+
 class ha_mroonga;
 
 /* structs */
@@ -811,7 +832,11 @@ public:
   bool check_and_repair(THD *thd);
   int analyze(THD* thd, HA_CHECK_OPT* check_opt);
   int optimize(THD* thd, HA_CHECK_OPT* check_opt);
-  bool is_fatal_error(int error_num, uint flags=0);
+  bool is_fatal_error(int error_num
+#ifdef MRN_HANDLER_IS_FATAL_ERROR_HAVE_FLAGS
+                      , uint flags
+#endif
+    ) mrn_override;
   bool check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                   uint table_changes);
 #ifdef MRN_HANDLER_HAVE_CHECK_IF_SUPPORTED_INPLACE_ALTER
@@ -834,8 +859,12 @@ public:
   void set_next_insert_id(ulonglong id);
   void get_auto_increment(ulonglong offset, ulonglong increment, ulonglong nb_desired_values,
                           ulonglong *first_value, ulonglong *nb_reserved_values);
-  void restore_auto_increment(ulonglong prev_insert_id);
-  void release_auto_increment();
+  void restore_auto_increment(ulonglong prev_insert_id)
+    MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE;
+#ifdef MRN_HANDLER_HAVE_RESTORE_AUTO_INCREMENT_NO_ARGUMENT
+  void restore_auto_increment() MRN_HANDLER_RESTORE_AUTO_INCREMENT_OVERRIDE;
+#endif
+  void release_auto_increment() MRN_HANDLER_RELEASE_AUTO_INCREMENT_OVERRIDE;
   int check_for_upgrade(HA_CHECK_OPT *check_opt);
 #ifdef MRN_HANDLER_HAVE_RESET_AUTO_INCREMENT
   int reset_auto_increment(ulonglong value);
@@ -878,7 +907,9 @@ protected:
 #ifdef MRN_HANDLER_HAVE_HA_INDEX_LAST
   int index_last(uchar *buf);
 #endif
-  bool primary_key_is_clustered();
+  bool primary_key_is_clustered()
+    MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST
+    mrn_override;
   bool is_fk_defined_on_table_or_index(uint index);
   char *get_foreign_key_create_info();
 #ifdef MRN_HANDLER_HAVE_GET_TABLESPACE_NAME
@@ -1607,8 +1638,16 @@ private:
   int storage_analyze(THD* thd, HA_CHECK_OPT* check_opt);
   int wrapper_optimize(THD* thd, HA_CHECK_OPT* check_opt);
   int storage_optimize(THD* thd, HA_CHECK_OPT* check_opt);
-  bool wrapper_is_fatal_error(int error_num, uint flags);
-  bool storage_is_fatal_error(int error_num, uint flags);
+  bool wrapper_is_fatal_error(int error_num
+#ifdef MRN_HANDLER_IS_FATAL_ERROR_HAVE_FLAGS
+                              , uint flags
+#endif
+    );
+  bool storage_is_fatal_error(int error_num
+#ifdef MRN_HANDLER_IS_FATAL_ERROR_HAVE_FLAGS
+                              , uint flags
+#endif
+    );
   bool wrapper_is_comment_changed(TABLE *table1, TABLE *table2);
   bool wrapper_check_if_incompatible_data(HA_CREATE_INFO *create_info,
                                           uint table_changes);
@@ -1732,6 +1771,10 @@ private:
                                   ulonglong *nb_reserved_values);
   void wrapper_restore_auto_increment(ulonglong prev_insert_id);
   void storage_restore_auto_increment(ulonglong prev_insert_id);
+#ifdef MRN_HANDLER_HAVE_RESTORE_AUTO_INCREMENT_NO_ARGUMENT
+  void wrapper_restore_auto_increment();
+  void storage_restore_auto_increment();
+#endif
   void wrapper_release_auto_increment();
   void storage_release_auto_increment();
   int wrapper_check_for_upgrade(HA_CHECK_OPT *check_opt);
@@ -1752,8 +1795,10 @@ private:
   bool wrapper_has_gap_locks() const;
   bool storage_has_gap_locks() const;
 #endif
-  bool wrapper_primary_key_is_clustered();
-  bool storage_primary_key_is_clustered();
+  bool wrapper_primary_key_is_clustered()
+    MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
+  bool storage_primary_key_is_clustered()
+    MRN_HANDLER_PRIMARY_KEY_IS_CLUSTERED_CONST;
   bool wrapper_is_fk_defined_on_table_or_index(uint index);
   bool storage_is_fk_defined_on_table_or_index(uint index);
   char *wrapper_get_foreign_key_create_info();
